@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2022 Synchrotron SOLEIL
+ * Copyright (C) 2003-2023 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -25,22 +25,56 @@
 #include "hkl/hkl-macros-private.h"
 #include "hkl-binoculars-private.h"
 
-hid_t create_dataspace_from_axes(const darray_axis *axes)
+static hid_t create_dataspace_from_axes(const darray_axis *axes)
 {
         HklBinocularsAxis *axis;
+        hid_t hid;
         darray(hsize_t) dims;
 
         darray_init(dims);
         darray_foreach(axis, *axes){
-                darray_append(dims, axis_size(axis));
+                if(axis_size(axis) > 1)
+			darray_append(dims, axis_size(axis));
         }
 
-        return H5Screate_simple(darray_size(dims), &darray_item(dims, 0), NULL);
+        hid = H5Screate_simple(darray_size(dims), &darray_item(dims, 0), NULL);
+        darray_free(dims);
+
+        return hid;
+}
+
+static herr_t save_string(hid_t group_id, const char *name, const char* config)
+{
+        hid_t dataset_id;
+        hid_t dataspace_id;
+        herr_t status;
+        hsize_t dims[] = {1};
+
+        hid_t filetype = H5Tcopy (H5T_C_S1);
+        status = H5Tset_size (filetype, strlen(config));
+        hid_t memtype = H5Tcopy (H5T_C_S1);
+        status = H5Tset_size (memtype, strlen(config));
+
+
+        dataspace_id = H5Screate_simple(1, dims, NULL);
+        dataset_id = H5Dcreate (group_id, "config",
+                                filetype, dataspace_id,
+                                H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+        status = H5Dwrite (dataset_id, memtype, H5S_ALL, H5S_ALL, H5P_DEFAULT, config);
+
+        status = H5Dclose(dataset_id);
+        status = H5Sclose(dataspace_id);
+        status = H5Tclose(memtype);
+        status = H5Tclose(filetype);
+
+        return status;
 }
 
 void hkl_binoculars_cube_save_hdf5(const char *fn,
+                                   const char *config,
                                    const HklBinocularsCube *self)
 {
+        double axis_idx = 0;
         hid_t file_id;
         hid_t groupe_id;
         hid_t groupe_axes_id;
@@ -54,6 +88,9 @@ void hkl_binoculars_cube_save_hdf5(const char *fn,
         groupe_id = H5Gcreate(file_id, "binoculars",
                               H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
 
+        // config
+        status = save_string(groupe_id, "config", config);
+
         // axes
         groupe_axes_id = H5Gcreate(groupe_id, "axes",
                                    H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
@@ -64,17 +101,26 @@ void hkl_binoculars_cube_save_hdf5(const char *fn,
                 hid_t dataset_id;
                 hsize_t dims[] = {6};
 
-                arr = hkl_binoculars_axis_array(axis);
-                dataspace_id = H5Screate_simple(ARRAY_SIZE(dims), dims, NULL);
-                dataset_id = H5Dcreate(groupe_axes_id, axis->name,
-                                       H5T_NATIVE_DOUBLE, dataspace_id,
-                                       H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
-                status = H5Dwrite(dataset_id, H5T_NATIVE_DOUBLE,
-                                  H5S_ALL, H5S_ALL,
-                                  H5P_DEFAULT, arr);
-                status = H5Dclose(dataset_id);
-                status = H5Sclose(dataspace_id);
-                free(arr);
+                if(axis_size(axis) > 1){
+                        arr = hkl_binoculars_axis_array(axis);
+                        /* the arr[0] contains the axis index expected
+                           by the binoculars gui. This value must
+                           start from zero, so compute this index
+                           using an external counter instead of using
+                           the original index stored in the axis
+                           array. */
+                        arr[0] = axis_idx++;
+                        dataspace_id = H5Screate_simple(ARRAY_SIZE(dims), dims, NULL);
+                        dataset_id = H5Dcreate(groupe_axes_id, axis->name,
+                                               H5T_NATIVE_DOUBLE, dataspace_id,
+                                               H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+                        status = H5Dwrite(dataset_id, H5T_NATIVE_DOUBLE,
+                                          H5S_ALL, H5S_ALL,
+                                          H5P_DEFAULT, arr);
+                        status = H5Dclose(dataset_id);
+                        status = H5Sclose(dataspace_id);
+                        free(arr);
+                }
         }
 
         status = H5Gclose(groupe_axes_id);
