@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2022 Synchrotron SOLEIL
+ * Copyright (C) 2003-2023 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -68,7 +68,7 @@ HKLAPI extern void hkl_binoculars_detector_2d_sixs_calibration(HklBinocularsDete
                                                                double *arr,
                                                                int width, int height,
                                                                int ix0, int iy0, double sdd,
-                                                               double detrot);
+                                                               double detrot, int normalize_flag);
 
 HKLAPI extern uint32_t *hkl_binoculars_detector_2d_fake_image_uint32(HklBinocularsDetectorEnum n,
                                                                      size_t *n_pixels);
@@ -92,7 +92,6 @@ typedef struct _HklBinocularsAxis HklBinocularsAxis;
 
 HKLAPI extern double *hkl_binoculars_axis_array(const HklBinocularsAxis *self);
 
-
 /*********/
 /* Space */
 /*********/
@@ -140,6 +139,7 @@ HKLAPI extern void hkl_binoculars_cube_add_space(HklBinocularsCube *self,
                                                  const HklBinocularsSpace *space);
 
 HKLAPI extern void hkl_binoculars_cube_save_hdf5(const char *fn,
+                                                 const char *config,
                                                  const HklBinocularsCube *self);
 
 HKLAPI extern void hkl_binoculars_cube_fprintf(FILE *f, const HklBinocularsCube *self);
@@ -148,140 +148,9 @@ HKLAPI extern void hkl_binoculars_cube_fprintf(FILE *f, const HklBinocularsCube
 /* Projections */
 /***************/
 
-/* class RealSpace(backend.ProjectionBase): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         pixels = pdataframe.pixels */
-/*         P = pdataframe.P */
-/*         timestamp = pdataframe.timestamp */
-
-/*         if P is not None: */
-/*             pixels_ = numpy.tensordot(P, pixels, axes=1) */
-/*         else: */
-/*             pixels_ = pixels */
-/*         x = pixels_[1] */
-/*         y = pixels_[2] */
-/*         if timestamp is not None: */
-/*             z = numpy.ones_like(x) * timestamp */
-/*         else: */
-/*             z = pixels_[0] */
-
-/*         return (x, y, z) */
-
-/*     def get_axis_labels(self): */
-/*         return ("x", "y", "z") */
-
-
-/* class Pixels(backend.ProjectionBase): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         pixels = pdataframe.pixels */
-
-/*         return numpy.meshgrid( */
-/*             numpy.arange(pixels[0].shape[1]), numpy.arange(pixels[0].shape[0]) */
-/*         ) */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "x", "y" */
-
-
-/* class QxQyIndexProjection(QxQyQzProjection): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         timestamp = pdataframe.timestamp */
-
-/*         qx, qy, qz = super(QxQyIndexProjection, self).project(index, pdataframe) */
-/*         return qx, qy, numpy.ones_like(qx) * timestamp */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Qx", "Qy", "t" */
-
-
-/* class QxQzIndexProjection(QxQyQzProjection): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         timestamp = pdataframe.timestamp */
-
-/*         qx, qy, qz = super(QxQzIndexProjection, self).project(index, pdataframe) */
-/*         return qx, qz, numpy.ones_like(qx) * timestamp */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Qx", "Qz", "t" */
-
-
-/* class QyQzIndexProjection(QxQyQzProjection): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         timestamp = pdataframe.timestamp */
-
-/*         qx, qy, qz = super(QyQzIndexProjection, self).project(index, pdataframe) */
-/*         return qy, qz, numpy.ones_like(qy) * timestamp */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Qy", "Qz", "t" */
-
-
-/* class QparQperIndexProjection(QparQperProjection): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         timestamp = pdataframe.timestamp */
-
-/*         qpar, qper = super(QparQperIndexProjection, self).project(index, pdataframe) */
-/*         return qpar, qper, numpy.ones_like(qpar) * timestamp */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Qpar", "Qper", "t" */
-
-
-/* class Stereo(QxQyQzProjection): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         qx, qy, qz = super(Stereo, self).project(index, pdataframe) */
-/*         q = numpy.sqrt(qx * qx + qy * qy + qz * qz) */
-/*         ratio = qz + q */
-/*         xp = qx / ratio */
-/*         yp = qy / ratio */
-/*         return q, xp, yp */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Q", "xp", "yp" */
-
-
-/* class QzPolarProjection(QxQyQzProjection): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         qx, qy, qz = super(QzPolarProjection, self).project(index, pdataframe) */
-/*         phi = numpy.rad2deg(numpy.arctan2(qx, qy)) */
-/*         q = numpy.sqrt(qx * qx + qy * qy + qz * qz) */
-/*         return phi, q, qz */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Phi", "Q", "Qz" */
-
-
-/* class QyPolarProjection(QxQyQzProjection): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         qx, qy, qz = super(QyPolarProjection, self).project(index, pdataframe) */
-/*         phi = numpy.rad2deg(numpy.arctan2(qz, qx)) */
-/*         q = numpy.sqrt(qx * qx + qy * qy + qz * qz) */
-/*         return phi, q, qy */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Phi", "Q", "Qy" */
-
-
-/* class QxPolarProjection(QxQyQzProjection): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         qx, qy, qz = super(QxPolarProjection, self).project(index, pdataframe) */
-/*         phi = numpy.rad2deg(numpy.arctan2(qz, -qy)) */
-/*         q = numpy.sqrt(qx * qx + qy * qy + qz * qz) */
-/*         return phi, q, qx */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Phi", "Q", "Qx" */
-
-
-/* class QIndex(Stereo): */
-/*     def project(self, index: int, pdataframe: PDataFrame) -> Tuple[ndarray]: */
-/*         timestamp = pdataframe.timestamp */
-
-/*         q, qx, qy = super(QIndex, self).project(index, pdataframe) */
-/*         return q, numpy.ones_like(q) * timestamp */
-
-/*     def get_axis_labels(self) -> Tuple[str]: */
-/*         return "Q", "Index" */
+/* subprojection for QCustom */
+/* q_index / tth_index  ( index = numero du scan) (optional) */
+/* q_timescan0 / tth_timescan0   (timescan0 = timestamp du 1er point du scan (constant pour 1 scan)) */
 
 /* angles */
 
@@ -305,49 +174,57 @@ HKLAPI extern HKL_BINOCULARS_SPACE_ANGLES_DECL(int32_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_ANGLES_DECL(uint16_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_ANGLES_DECL(uint32_t);
 
-/* qparqper */
+/* qcustom */
 
-#define HKL_BINOCULARS_SPACE_QPARQPER_DECL(image_t)			\
-        void hkl_binoculars_space_qparqper_ ## image_t (HklBinocularsSpace *space, \
-                                                        const HklGeometry *geometry, \
-                                                        const image_t *image, \
-                                                        size_t n_pixels, \
-                                                        double weight,  \
-                                                        const double *pixels_coordinates, \
-                                                        size_t pixels_coordinates_ndim, \
-                                                        const size_t *pixels_coordinates_dims, \
-                                                        const double *resolutions, \
-                                                        size_t n_resolutions, \
-                                                        const uint8_t *masked, \
-                                                        HklBinocularsSurfaceOrientationEnum surf, \
-                                                        const HklBinocularsAxisLimits **limits, \
-                                                        size_t n_limits)
+typedef enum _HklBinocularsQCustomSubProjectionEnum
+{
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QX_QY_QZ = 0,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TTH_TIMESTAMP,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TIMESTAMP,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPAR_QPER_TIMESTAMP,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPAR_QPER,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QX,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QY,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QZ,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_STEREO,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_DELTALAB_GAMMALAB_SAMPLEAXIS,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_X_Y_Z,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Y_Z_TIMESTAMP,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_QPAR_QPER,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPARS_QPER_TIMESTAMP,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPAR_QPER_SAMPLEAXIS,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_SAMPLEAXIS_TTH,
+        /* Add new your subprojection in the same order than the haskell order here */
+        HKL_BINOCULARS_QCUSTOM_NUM_SUBPROJECTIONS,
+} HklBinocularsQCustomSubProjectionEnum;
 
-HKLAPI extern HKL_BINOCULARS_SPACE_QPARQPER_DECL(int32_t);
-HKLAPI extern HKL_BINOCULARS_SPACE_QPARQPER_DECL(uint16_t);
-HKLAPI extern HKL_BINOCULARS_SPACE_QPARQPER_DECL(uint32_t);
+#define HKL_BINOCULARS_SPACE_QCUSTOM_DECL(image_t)			\
+        void hkl_binoculars_space_qcustom_ ## image_t (HklBinocularsSpace *space, \
+                                                       const HklGeometry *geometry, \
+                                                       const image_t *image, \
+                                                       size_t n_pixels, \
+                                                       double weight,   \
+                                                       const double *pixels_coordinates, \
+                                                       size_t pixels_coordinates_ndim, \
+                                                       const size_t *pixels_coordinates_dims, \
+                                                       const double *resolutions, \
+                                                       size_t n_resolutions, \
+                                                       const uint8_t *masked, \
+                                                       HklBinocularsSurfaceOrientationEnum surf, \
+                                                       const HklBinocularsAxisLimits **limits, \
+                                                       size_t n_limits, \
+                                                       double timestamp, \
+                                                       const HklBinocularsQCustomSubProjectionEnum subprojection, \
+                                                       double uqx,      \
+                                                       double uqy,      \
+                                                       double uqz,      \
+                                                       const char *sample_axis \
+                )
 
-/* qxqyqz */
 
-#define HKL_BINOCULARS_SPACE_QXQYQZ_DECL(image_t)			\
-        void hkl_binoculars_space_qxqyqz_ ## image_t (HklBinocularsSpace *space, \
-                                                      const HklGeometry *geometry, \
-                                                      const image_t *image, \
-                                                      size_t n_pixels,  \
-                                                      double weight,    \
-                                                      const double *pixels_coordinates, \
-                                                      size_t pixels_coordinates_ndim, \
-                                                      const size_t *pixels_coordinates_dims, \
-                                                      const double *resolutions, \
-                                                      size_t n_resolutions, \
-                                                      const uint8_t *masked, \
-                                                      HklBinocularsSurfaceOrientationEnum surf, \
-                                                      const HklBinocularsAxisLimits **limits, \
-                                                      size_t n_limits)
-
-HKLAPI extern HKL_BINOCULARS_SPACE_QXQYQZ_DECL(int32_t);
-HKLAPI extern HKL_BINOCULARS_SPACE_QXQYQZ_DECL(uint16_t);
-HKLAPI extern HKL_BINOCULARS_SPACE_QXQYQZ_DECL(uint32_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_QCUSTOM_DECL(int32_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_QCUSTOM_DECL(uint16_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_QCUSTOM_DECL(uint32_t);
 
 /* hkl */
 
@@ -371,6 +248,30 @@ HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(int32_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(uint16_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(uint32_t);
 
+/* test */
+
+#define HKL_BINOCULARS_SPACE_TEST_DECL(image_t)			\
+        void hkl_binoculars_space_test_ ## image_t (HklBinocularsSpace *space, \
+                                                    const HklGeometry *geometry, \
+                                                    const HklSample *sample, \
+                                                    const image_t *image, \
+                                                    size_t n_pixels,    \
+                                                    double weight,      \
+                                                    const double *pixels_coordinates, \
+                                                    size_t pixels_coordinates_ndim, \
+                                                    const size_t *pixels_coordinates_dims, \
+                                                    const double *resolutions, \
+                                                    size_t n_resolutions, \
+                                                    const uint8_t *masked, \
+                                                    const HklBinocularsAxisLimits **limits, \
+                                                    size_t n_limits)
+
+
+HKLAPI extern HKL_BINOCULARS_SPACE_TEST_DECL(int32_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_TEST_DECL(uint16_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_TEST_DECL(uint32_t);
+
+
 G_END_DECLS
 
 #endif
