@@ -3,7 +3,7 @@
 {-# LANGUAGE StandaloneDeriving #-}
 
 {-
-    Copyright  : Copyright (C) 2014-2022 Synchrotron SOLEIL
+    Copyright  : Copyright (C) 2014-2023 Synchrotron SOLEIL
                                          L'Orme des Merisiers Saint-Aubin
                                          BP 48 91192 GIF-sur-YVETTE CEDEX
     License    : GPL3+
@@ -18,27 +18,36 @@ module Hkl.Binoculars.Common
   , InputFn(..)
   , addSpace
   , chunk
+  , cclip
   , clength
   , mkCube'
   , toList
   , withCubeAccumulator
   ) where
 
-import           Control.Exception     (bracket)
-import           Data.Array.Repa       (Shape)
-import           Data.IORef            (IORef, newIORef, readIORef)
-import           Foreign.ForeignPtr    (withForeignPtr)
-import           Foreign.Marshal.Array (withArrayLen)
-import           Path                  (Abs, File, Path, fromAbsFile)
-import           Text.Printf           (printf)
+import           Control.Exception          (bracket)
+import           Data.Array.Repa            (Shape)
+import           Data.IORef                 (IORef, newIORef, readIORef)
+import           Foreign.ForeignPtr         (withForeignPtr)
+import           Foreign.Marshal.Array      (withArrayLen)
+import           Path                       (Abs, File, Path, fromAbsFile)
+import           Text.Printf                (printf)
 
+import           Hkl.Binoculars.Config
+import           Hkl.Binoculars.Projections
 import           Hkl.C.Binoculars
 import           Hkl.Image
-import           Hkl.Orphan            ()
+import           Hkl.Orphan                 ()
 
 data Chunk n a = Chunk !a !n !n
 deriving instance (Show n, Show a) => Show (Chunk n a)
 
+cclip :: (Num n, Ord n) => n -> n -> Chunk n a -> Chunk n a
+cclip skipL skipH (Chunk a l h) = let nl = if l + skipL > h then l else l + skipL
+                                      nh = if h - skipH < l then h else h - skipH
+                                  in Chunk a nl nh
+{-# SPECIALIZE cclip :: Int -> Int -> Chunk Int FilePath -> Chunk Int FilePath #-}
+
 clength :: Num n => Chunk n a -> n
 clength (Chunk _ l h) = h - l + 1
 {-# SPECIALIZE clength :: Chunk Int FilePath -> Int  #-}
@@ -64,19 +73,20 @@ chunk target = go target target
     cons1 x cs = (x : Prelude.head cs) : tail cs
 
     golast tgt gap x =
-      if | cweight x <= gap         -> [[x]]
+      if | 0 == gap                 -> [[x]]
+         | cweight x <= gap         -> [[x]]
          | (x1, x2) <- csplit x gap -> [x1] : golast tgt tgt x2
 
 {-# SPECIALIZE chunk :: Int -> [Chunk Int FilePath] -> [[Chunk Int FilePath]]  #-}
 
 toList :: InputFn -> [FilePath]
-toList (InputFn f)           = [f]
-toList (InputRange tmpl f t) = [printf tmpl i | i <- [f..t]]
-toList (InputList fs)        = map fromAbsFile fs
+toList (InputFn f)              = [f]
+toList (InputFn'Range tmpl f t) = [printf tmpl i | i <- [f..t]]
+toList (InputFn'List fs)        = map fromAbsFile fs
 
 --  DataFrameSpace
 
-data DataFrameSpace sh = DataFrameSpace Image (Space sh) Double
+data DataFrameSpace sh = DataFrameSpace Image (Space sh) Attenuation
   deriving Show
 
 --  Create the Cube
@@ -101,8 +111,8 @@ addSpace (DataFrameSpace _ (Space fs) _) (Cube fp) =
 type Template = String
 
 data InputFn = InputFn FilePath
-             | InputRange Template Int Int
-             | InputList [Path Abs File]
+             | InputFn'Range Template Int Int
+             | InputFn'List [Path Abs File]
   deriving Show
 
 withCubeAccumulator :: Shape sh => Cube sh -> (IORef (Cube sh)  -> IO ()) -> IO (Cube sh)
